//! bare6.rs
//! Simple bare metal application
//!
#![feature(used)]
#![feature(use_nested_groups)]
#![no_std]

extern crate cortex_m;
extern crate cortex_m_rt;

#[macro_use]
extern crate cortex_m_debug;

use cortex_m::{asm::bkpt, peripheral::DWT};

// uses the DWT.CYCNT
// doc: ARM trm_100166_0001_00_en.pdf, chapter 9.2
// we use the `cortex-m` abstraction
fn wait_cycles(nr_cycles: u32) {
    unsafe {
        let t = (*DWT.get()).cyccnt.read().wrapping_add(nr_cycles);
        while ((*DWT.get()).cyccnt.read().wrapping_sub(t) as i32) < 0 {}
    }
}

fn main() {
    unsafe { (*DWT.get()).enable_cycle_counter() };
    let mut i = 0;
    loop {
        ipln!("tick {}", i);
        wait_cycles(64_000_000);
        i += 1; // this will eventually cause a panic, when the counter wraps.
    }
}

// As we are not using interrupts, we just register a dummy catch all handler
#[link_section = ".vector_table.interrupts"]
#[used]
static INTERRUPTS: [extern "C" fn(); 240] = [default_handler; 240];

extern "C" fn default_handler() {
    cortex_m::asm::bkpt();
}
